# frozen_string_literal: true

module  Drillbit
class   AcceptHeader
  attr_accessor :application,
                :raw_accept_header

  def initialize(application:, header:)
    self.application       = application
    self.raw_accept_header = header || ''
  end

  def version
    accept_header_data[2]
  end

  def content_type
    accept_header_data[1]
  end

  def valid?
    !invalid?
  end

  def invalid?
    accept_header_data.nil?
  end

  def to_s
    raw_accept_header
  end

  private

  def accept_header_data
    raw_accept_header.match(accept_header_format)
  end

  # rubocop:disable Metrics/LineLength
  def accept_header_format
    %r{(?:(?<=\A)|(?<=,))application/#{application_vendor}(?:\+(\w+))?(?:;version=(#{version_format}))?(?:(?=\z)|(?=,))}
  end
  # rubocop:enable Metrics/LineLength

  def application_vendor
    "vnd\\.#{application}"
  end

  def version_format
    '\\d+(?:\\.\\d+){0,2}(?:beta(?:\\d*))?'
  end
end
end
