# frozen_string_literal: true

require 'drillbit/authorizers/parameters'
require 'drillbit/errors/unpermitted_inclusions'

module  Drillbit
module  Authorizers
class   Parameters
class   Inclusions < Authorizers::Parameters
  attr_writer :authorized_inclusions

  def call
    fail Errors::UnpermittedInclusions.new(inclusions: requested_inclusions) \
      if inclusion_differences.any?

    requested_inclusions
  end

  def authorized_inclusions
    @authorized_inclusions ||= []
  end

  private

  def add_includable_resource(resource_name)
    authorized_inclusions << resource_name
  end

  def add_includable_resources(*resource_names)
    resource_names.each do |resource_name|
      add_includable_resource(resource_name)
    end
  end

  def requested_inclusions
    @requested_inclusions ||= params
                                .fetch(:include, '')
                                .split(',')
                                .map(&:to_sym)
  end

  def inclusion_differences
    @inclusion_differences ||= requested_inclusions - authorized_inclusions
  end
end
end
end
end
