# frozen_string_literal: true

module  Drillbit
  class Configuration
    attr_writer :allowed_api_subdomains,
                :allowed_subdomains,
                :available_token_roles,
                :default_api_version,
                :default_token_audience,
                :default_token_expiration_in_minutes,
                :default_token_issuer,
                :default_token_roles,
                :default_token_subject,
                :token_private_key,
                :token_type

    attr_accessor :application_name

    def to_h
      {
        allowed_api_subdomains:              allowed_api_subdomains,
        allowed_subdomains:                  allowed_subdomains,
        application_name:                    application_name,
        default_api_version:                 default_api_version,
        default_token_issuer:                default_token_issuer,
        default_token_subject:               default_token_subject,
        default_token_expiration_in_minutes: token_expiration_in_minutes,
      }
    end

    def allowed_subdomains
      @allowed_subdomains || ['api']
    end

    def allowed_api_subdomains
      @allowed_api_subdomains || ['api']
    end

    def default_token_audience
      @default_token_audience || 'public'
    end

    def available_token_roles
      @available_token_roles || %w{standard admin password_reset email_verification}
    end

    def default_api_version
      @default_api_version || '1'
    end

    def default_token_roles
      @default_token_roles || %w{standard}
    end

    def default_token_issuer
      @default_token_issuer || 'drillbit'
    end

    def default_token_subject
      @default_token_subject || 'User'
    end

    def default_token_expiration_in_minutes
      @default_token_expiration_in_minutes || (7 * 24 * 60)
    end

    def token_private_key
      return unless @token_private_key
      return @token_private_key if @token_private_key.respond_to?(:sign)

      OpenSSL::PKey::RSA.new(@token_private_key)
    end

    def token_type
      @token_type || 'JWE'
    end
  end

  def self.configure
    yield configuration
  end

  def self.configuration
    @configuration ||= Configuration.new
  end
end
