# frozen_string_literal: true

require 'drillbit/resource/processors/filtering'
require 'drillbit/resource/processors/sorting'
require 'drillbit/resource/processors/paging'
require 'drillbit/resource/processors/indexing'

module  Drillbit
module  Resource
class   Model
  DEFAULT_PROCESSORS = %w{filtering sorting paging indexing}.freeze

  attr_accessor :resource,
                :parameters
  attr_reader   :processors

  def initialize(resource:, parameters:, **options)
    self.resource   = resource
    self.parameters = parameters.dup
    self.processors = options.fetch(:processors, DEFAULT_PROCESSORS)
  end

  def processed
    @processed ||= processors.inject(resource) do |processed_resource, processor|
      processor.processed(processed_resource, parameters)
    end
  end

  def meta
    @meta ||= processors.inject({}) do |metadata, processor|
      metadata.merge processor.meta(processed, parameters)
    end
  end

  def processors=(other)
    @processors = other.map do |processor|
      Object.const_get "::Drillbit::Resource::Processors::#{processor.capitalize}"
    end
  end
end
end
end
